<?php

namespace Teknisa\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use setasign\Fpdi\Fpdi;

use Teknisa\Util\FPDF_Merge;
use Teknisa\Util\Utilities;
use Teknisa\Models\Danfe;
use Teknisa\Models\Damdfe;
use Teknisa\Models\Dacte;

class DanfeController {
    
    private $erros      = array();
    private $sucessos   = array();
    
    /*
    * Chama o metódo que gera uma danfe a partir de um xml
    *
    * @param Zeedhi\Framework\DTO\Request\Row $row
    * @param Zeedhi\Framework\DTO\Response $response
    **/    
    public function callGerarDanfe(Request\Row $request, Response $response) {
        $row = $request->getRow();

        $excpectFields = array("FILES", "DADOSEMITENTE", "MERGEPDF");
        $camposProduto = array("vBCFCP" => "BcFcp", "pFCP" => "pFcp", "vFCP" => "vFcp", "pRedBC" => "pRedBC", "pMVAST" => "IVA/MVA", "pICMSST" => "pIcmsSt", "vBCST" => "BcICMSSt", "pICMSST" => "pICMSST", "vICMSST" => "vICMSSt",
                                     "vBCFCPST" => "BcFCPSt", "pFCPST" => "pFCPSt", "vFCPST" => "vFCPSt", "pFCPUFDest" => "pFCPUFDest", "pICMSUFDest" => "pICMSUFDest", "pICMSInterPart" => "pICMSInterPart", 
                                     "vFCPUFDest" => "vFCPUFDest", "vICMSUFDest" => "vICMSUFDest", "vICMSUFRemet" => "vICMSUFRemet", "cEAN" => "cEAN", "cEANTrib" => "cEANTrib", "vUnTrib" => "vUnTrib",
                                     "qTrib" => "qTrib", "uTrib" => "uTrib");
        
        
        Utilities::validateExpectedFields($row, $excpectFields, "Request");

        $xmls           = $row["FILES"];
        $dadosEmitente  = $row["DADOSEMITENTE"];
        $mergePDF       = $row["MERGEPDF"]["MERGEPDF"];
        $logo           = ($row["LOGO"]) ? $row["LOGO"]["LOGO"] : '';
        $camposProduto  = (isset($row["CAMPOSPRODUTO"])) ? $row["CAMPOSPRODUTO"]["CAMPOSPRODUTO"] : $camposProduto;
        
        try {
            $zip = $this->gerarDanfes($dadosEmitente, $xmls, $mergePDF, $logo, $camposProduto);
        }
        catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
            return false;
        }

        if($this->sucessos) {
            $response->addDataSet(new DataSet("danfe", array(base64_encode($zip))));
        }
        else {
            $response->addDataSet(new DataSet("danfe", array()));
        }
        $response->addDataSet(new DataSet("erros", $this->erros));
        $response->addDataSet(new DataSet("successos", $this->sucessos));
    }
    
     /*
    * Chama o metódo que gera uma damdfe a partir de um xml
    *
    * @param Zeedhi\Framework\DTO\Request\Row $row
    * @param Zeedhi\Framework\DTO\Response $response
    **/    
    public function callGerarDamdfe(Request\Row $request, Response $response) {
        $row = $request->getRow();

        $excpectFields = array("FILES", "DADOSEMITENTE", "MERGEPDF");
        
        Utilities::validateExpectedFields($row, $excpectFields, "Request");

        $xmls           = $row["FILES"];
        $dadosEmitente  = $row["DADOSEMITENTE"];
        $mergePDF       = $row["MERGEPDF"]["MERGEPDF"];
        $logo           = ($row["LOGO"]) ? $row["LOGO"]["LOGO"] : '';
        
        try {
            $zip = $this->gerarDanfes($dadosEmitente, $xmls, $mergePDF, $logo, array(), 'DAMDFE');
        }
        catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
            return false;
        }

        if($this->sucessos) {
            $response->addDataSet(new DataSet("damdfe", array(base64_encode($zip))));
        }
        else {
            $response->addDataSet(new DataSet("damdfe", array()));
        }
        $response->addDataSet(new DataSet("erros", $this->erros));
        $response->addDataSet(new DataSet("successos", $this->sucessos));
    }
    
    /*
    * Chama o metódo que gera uma damdfe a partir de um xml
    *
    * @param Zeedhi\Framework\DTO\Request\Row $row
    * @param Zeedhi\Framework\DTO\Response $response
    **/    
    public function callGerarDacte(Request\Row $request, Response $response) {
        $row = $request->getRow();

        $excpectFields = array("FILES", "DADOSEMITENTE", "MERGEPDF");
        
        Utilities::validateExpectedFields($row, $excpectFields, "Request");

        $xmls           = $row["FILES"];
        $dadosEmitente  = $row["DADOSEMITENTE"];
        $mergePDF       = $row["MERGEPDF"]["MERGEPDF"];
        $logo           = ($row["LOGO"]) ? $row["LOGO"]["LOGO"] : '';
        
        try {
            $zip = $this->gerarDanfes($dadosEmitente, $xmls, $mergePDF, $logo, array(), 'DACTE');
        }
        catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
            return false;
        }

        if($this->sucessos) {
            $response->addDataSet(new DataSet("dacte", array(base64_encode($zip))));
        }
        else {
            $response->addDataSet(new DataSet("dacte", array()));
        }
        $response->addDataSet(new DataSet("erros", $this->erros));
        $response->addDataSet(new DataSet("successos", $this->sucessos));
    }
        
    /*
    * Gera as danfes e compacta a partir dos arquivos XML informados
    *
    * @param array $dadosEmitente
    * @param array $xmls
    * @param boolean $mergePDF
    * @param string  $logo
    **/
    private function gerarDanfes($dadosEmitente, $xmls, $mergePDF = true, $logo, $camposProduto, $type = "DANFE") {
        $baseDir              = sys_get_temp_dir();
        $inscricaoJuridica = ($dadosEmitente["INSCRICAOJURIDICA"]) ? $dadosEmitente["INSCRICAOJURIDICA"] : '';
        $dirLogo = '';
        
        if($logo) {
            
            $dirLogo = "$baseDir/$inscricaoJuridica.jpg";
            
            if(is_file($dirLogo)) {
                unlink($dirLogo);
            } 
            
            $file = imagecreatefromstring(base64_decode($logo));
            imagejpeg($file, $dirLogo);
        }

        $zipBase64  = "";
        $random     = substr(md5(microtime()),1,rand(8,12));
        $zipName    = "$baseDir/$random-$inscricaoJuridica.zip";
        $result     = array("erros" => array(), "sucesso" => array(), "zipBase64" => $zipBase64);
        $zip        = new \ZipArchive;
        
        if(is_file($zipName)) {
            $zip->open($zipName, \ZipArchive::OVERWRITE);
        }
        else {
            $zip->open($zipName, \ZipArchive::CREATE);
        }
        
        if($mergePDF == true) {
            $fpdi = new Fpdi();
        } 
        
        foreach($xmls as $file) {
            $chave = $file["chave"];
            $xml   = $file["xml"];
            switch($type) {
                case 'DANFE':
                    $file = $this->gerarDanfe($chave, $xml, $inscricaoJuridica, $baseDir, $dirLogo, $mergePDF, $camposProduto);
                    break;
                case 'DAMDFE':
                    $file = $this->gerarDamdfe($chave, $xml, $inscricaoJuridica, $baseDir, $dirLogo, $mergePDF, $camposProduto);
                    break;
                case 'DACTE':
                    $file = $this->gerarDacte($chave, $xml, $inscricaoJuridica, $baseDir, $dirLogo, $mergePDF, $camposProduto);
                    break;
            }


            if($file) {
                if($mergePDF === false) {
                    switch($type) {
                        case 'DANFE':
                            $zip->addFile($file["dirFile"], $chave.'-nfe.pdf');
                            break;
                        case 'DAMDFE':
                            $zip->addFile($file["dirFile"], $chave.'-mdfe.pdf');
                            break;
                        case 'DACTE':
                            $zip->addFile($file["dirFile"], $chave.'-cte.pdf');
                            break;
                    }

                }
                else {
                    $tpImp = isset($file["tpImp"]) ? $file["tpImp"] : "P";
                    $file = $file["dirFile"];
                    
                    $pagecount = $fpdi->setSourceFile($file);  
                    for($i = 1; $i <= $pagecount; $i++){
                        $fpdi->AddPage($tpImp);  
                        $tplidx = $fpdi->importPage($i, '/MediaBox');
                        $fpdi->useTemplate($tplidx); 
                    }
                }
            }

        }
        
        $fileName = '';
        if($mergePDF === true) {
            $fileName = '/PDF'.date("dmYHis").$inscricaoJuridica.'.pdf';
            $fpdi->Output($baseDir.$fileName, 'F');
            $zip->addFile($baseDir.$fileName, $fileName);
        }
            
        $caminhoZip = $zip->filename;
        
        $zip->close();
        
        if(is_file($caminhoZip)) {
            $zipBase64 = file_get_contents($caminhoZip);
            $zipBase64 = base64_encode($zipBase64);
            $zip = file_get_contents($caminhoZip);
        }
        else {
            $zip = '';
        }
        
        if($this->sucessos) {
            $zip = file_get_contents($caminhoZip);
            $this->deleteFiles($baseDir, $dirLogo, $xmls, $zipName, $fileName);
            return $zip;
        }
        else {
            $this->deleteFiles($baseDir, $dirLogo, $xmls, $zipName, $fileName);
            return '';
        }
    }
    
    /*
    * Gera uma danfe a partir do seu arquivo xml
    *
    * @param string $chave
    * @param string $xml
    * @param string $inscricaoJuridica
    * @param string $baseDir
    * @param string $dirLogo
    * @param boolean $mergePDF
    * @param array  $camposProduto
    **/
    private function gerarDanfe($chave, $xml, $inscricaoJuridica, $baseDir, $dirLogo, $mergePDF, $camposProduto) {
        $format = 'L';
        $dirFile = "$baseDir/$chave-nfe.pdf";
        
        try {
            
            $danfe = new Danfe($xml, $format, 'A4',  $dirLogo, 'I', '');
            $danfe->setCamposProduto($camposProduto);
            $danfe->montaDANFE();
            
            $danfe->printDocument($dirFile, 'F');
            $this->sucessos[] = $chave;
            
            return array( 
                "dirFile" => $dirFile,
                "tpImp" => $danfe->getTpImp()
            );
        }
        catch(\Exception $e) {

            $this->erros[] = array("chave" => $chave, "erro" => array($e->getMessage()));
            return null;
        }
        
    }
    
    /*
    * Gera uma damdfe a partir do seu arquivo xml
    *
    * @param string $chave
    * @param string $xml
    * @param string $inscricaoJuridica
    * @param string $baseDir
    * @param string $dirLogo
    * @param boolean $mergePDF
    * @param array  $camposProduto
    **/
    private function gerarDamdfe($chave, $xml, $inscricaoJuridica, $baseDir, $dirLogo, $mergePDF, $camposProduto) {
        $format = 'P';
        $dirFile = "$baseDir/$chave-mdfe.pdf";
        
        try {
            
            $damdfe = new Damdfe($xml, $format, 'A4',  $dirLogo, 'I', '');

            $damdfe->buildMDFe();
            
            $damdfe->printMDFe($dirFile, 'F');
            $this->sucessos[] = $chave;

            return array( 
                "dirFile" => $dirFile,
                "tpImp" => $damdfe->getTpImp()
            );
        }
        catch(\Exception $e) {

            $this->erros[] = array("chave" => $chave, "erro" => array($e->getMessage()));
            return null;
        }   
    }
    
    /*
    * Gera uma dacte a partir do seu arquivo xml
    *
    * @param string $chave
    * @param string $xml
    * @param string $inscricaoJuridica
    * @param string $baseDir
    * @param string $dirLogo
    * @param boolean $mergePDF
    * @param array  $camposProduto
    **/
    private function gerarDacte($chave, $xml, $inscricaoJuridica, $baseDir, $dirLogo, $mergePDF, $camposProduto) {
        $format = 'L';
        $dirFile = "$baseDir/$chave-cte.pdf";
        
        try {
            
            $dacte = new Dacte($xml, $format, 'A4',  $dirLogo, 'I', '');

            $dacte->monta();
            
            $dacte->printDocument($dirFile, 'F');
            $this->sucessos[] = $chave;

            return array( 
                "dirFile" => $dirFile,
                "tpImp" => $dacte->getTpImp()
            );
        }
        catch(\Exception $e) {

            $this->erros[] = array("chave" => $chave, "erro" => array($e->getMessage()));
            return null;
        }   
    }
    
    /*
    * Remove os arquivos das danfe's geradas do servidor
    *
    * @param string $baseDir
    * @param string $dirLogo
    * @param array  $files
    * @param string $zipName
    * @param string $fileName
    **/
    private function deleteFiles($baseDir, $dirLogo, $files, $zipName, $fileName = '') {
        if(!empty($this->sucessos)) {
            foreach($files as $chave => $file) {
                $dir_file = "$baseDir$chave-nfe.pdf";
                if(is_file($dir_file)) {
                  //  unlink($dir_file);
                }
            }
            
            if(is_file($zipName)) {
                unlink($zipName);
            }
    
            if(is_file($baseDir.$fileName)) {
                unlink($baseDir.$fileName);
            }
        }
        else {
           $result["zipBase64"] = "";
        }
        if(is_file($dirLogo)) {
          unlink($dirLogo); 
        }
    }
}